import pytest
from dingus import Dingus
from munch import Munch

from jobtriggers import module_build_msg


@pytest.mark.usefixtures('prepare')
class TestModuleBuildDoneJobConsumer():

    @pytest.fixture
    def prepare(self):
        self.ref_task = 'module_task1'
        self.ref_name = 'testmodule'
        self.ref_stream = 'master'
        self.ref_version = '20170325004958'
        self.ref_item = '%s-%s-%s' % (self.ref_name, self.ref_stream, self.ref_version)

        self.ref_message = self._create_msg(self.ref_name, self.ref_stream, self.ref_version)

        stub_hub = Munch(config=Munch(get=0))

        self.helper = module_build_msg.ModuleBuildDoneJobConsumer(stub_hub)
        self.helper.trigger.runner = Dingus()

        module_build_msg.config.trigger_rules_template = """---
- when: {message_type: ModuleBuildComplete}
  do:
    - {tasks: [%s]}
""" % self.ref_task

        module_build_msg.config.job_logging = False
        module_build_msg.config.valid_arches = ['x86_64']
        module_build_msg.config.default_host_arch = 'noarch'

    def _create_msg(self, name, stream, version, state_name='done'):
        return Munch(body={
          "source_name": "datanommer",
          "i": 20,
          "timestamp": 1490404041.0,
          "msg_id": "2017-683a79ca-b143-49df-a659-0d76a65c4a91",
          "topic": "org.fedoraproject.prod.mbs.module.state.change",
          "source_version": "0.6.5",
          "msg": {
            "state_reason": "null",
            "component_builds": [
              2653,
              2654,
              2655
            ],
            "name": name,
            "stream": stream,
            "time_submitted": 1490403045.0,
            "tasks": {
              "rpms": {
                "ed": {
                  "state": 1,
                  "nvr": "ed-1.14.1-2.module_b98abea8",
                  "task_id": 18582394,
                  "state_reason": ""
                },
                "module-build-macros": {
                  "state": 1,
                  "nvr": "module-build-macros-0.1-1.module_b98abea8",
                  "task_id": 18582338,
                  "state_reason": ""
                },
                "mksh": {
                  "state": 1,
                  "nvr": "mksh-54-2.module_b98abea8",
                  "task_id": 18582475,
                  "state_reason": ""
                }
              }
            },
            "state_url": "http://localhost/module-build-service/1/module-builds/75",
            "time_modified": 1490404041.0,
            "scmurl": "git://pkgs.fedoraproject.org/modules/testmodule.git?#69d45ad",
            "state": 3,
            "modulemd": "random data\n",
            "time_completed": 1490404041.0,
            "version": version,
            "owner": "psabata",
            "state_trace": [
              {
                "reason": "null",
                "time": 1490403047.0,
                "state": 1,
                "state_name": "wait"
              },
              {
                "reason": "null",
                "time": 1490403073.0,
                "state": 2,
                "state_name": "build"
              },
              {
                "reason": "null",
                "time": 1490403073.0,
                "state": 2,
                "state_name": "wait"
              }
            ],
            "id": 75,
            "state_name": state_name
          }
        })

    def test_consume(self):
        self.helper.consume(self.ref_message)

        runner_calls = self.helper.trigger.runner.trigger_job.calls()

        assert len(runner_calls) == 1

        assert runner_calls[0][1] == (
            self.ref_item,
            module_build_msg.ITEM_TYPE,
            self.ref_task,
            'x86_64',
            'noarch',
        )

    def test_consume_not_done(self):
        ref_message = self._create_msg(self.ref_name, self.ref_stream,
                                       self.ref_version, state_name='failed')
        self.helper.consume(ref_message)

        runner_calls = self.helper.trigger.runner.trigger_job.calls()

        assert len(runner_calls) == 0
