import pytest
from dingus import Dingus
from munch import Munch
from copy import deepcopy
import fedmsg.encoding
import fedmsg.consumers

from jobtriggers import taskotron_result_missing_msg


@pytest.mark.usefixtures('prepare')
class TestTaskotronResultMissingJobConsumer():

    @pytest.fixture
    def prepare(self, monkeypatch):
        self.ref_item = 'ref_item'
        self.ref_item_type = 'ref_item_type'
        self.ref_testcase = 'ref.testcase.subcase'
        self.ref_arch = 'ref_arch'
        self.ref_task = 'ref_task'
        self.ref_validarches = ['i386', 'x86_64']

        self._create_msg(
            self.ref_item, self.ref_item_type, self.ref_testcase, self.ref_arch)

        self.ref_data = {
            "_msg": {},
            "message_type": "TaskotronResultMissing",
            "item": self.ref_item,
            "item_type": self.ref_item_type,
            "testcase": self.ref_testcase,
            "arches": [self.ref_arch],
        }

        stub_hub = Munch(config=Munch(get=0))
        self.helper = taskotron_result_missing_msg.TasktoronResultMissingJobConsumer(stub_hub)

        self.helper.trigger.runner = Dingus()
        taskotron_result_missing_msg.config.trigger_rules_template = """---
- do:
  - {tasks: [ref_task], arches: ${arches}}
  when: {message_type: TaskotronResultMissing, testcase: {$regex: '/^ref.testcase(\..+)*/'}}
"""

        taskotron_result_missing_msg.config.valid_arches = self.ref_validarches
        taskotron_result_missing_msg.config.job_logging = False
        taskotron_result_missing_msg.config.default_host_arch = 'noarch'

    def _create_msg(self, ref_item, ref_item_type, ref_testcase, ref_arch):
        self.ref_message = Munch(body='{"i": 1,\
                                        "msg": {"item": "%s",\
                                                "item_type": "%s",\
                                                "testcase": "%s",\
                                                "arch": "%s"},\
                                        "timestamp": 1359603469.21164,\
                                        "topic": "org.fedoraproject.prod.taskotron.result.missing",\
                                        "username": "apache"}' %
                                 (ref_item, ref_item_type, ref_testcase, ref_arch))

    def test_consume(self):
        self.helper.consume(self.ref_message)

        runner_calls = self.helper.trigger.runner.trigger_job.calls()

        assert len(runner_calls) == 1

        assert runner_calls[0][1] == (self.ref_item, self.ref_item_type,
                                      self.ref_task, self.ref_arch, 'noarch')

